#!/usr/bin/env python
# SAT (Simple scAnning Tool)
# http://sat.berlios.de/
# Copyright (C) 2005 azurIt (azurit@pobox.sk, azurIt@IRCnet)
#
# LICENSE:
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
################################################################################
import select, telnetlib, threading, socket, re, signal, cPickle, Queue
from os import linesep
from fnmatch import translate
from sys import exit, argv, stdout
from thread import error as ThreadError
from time import strftime, time, sleep, localtime

def open_file(name, mode):
	try:
		file_obj = open(name, mode)
	except IOError, error:
		return error
	else:
		return file_obj

def test_ipv4(ip):
	ip = ip.replace(".", " ").split()
	if len(ip) != 4:
		return 0
	try:
		for i in xrange(4):
			ip[i] = int(ip[i])
			if ip[i] < 0 or ip[i] > 255:
				return 0
	except ValueError:
		return 0
	return 1

def sig_hand(signum, frame):
	sat.die = 1

# this is here for signals on Windows compatibility
# (it seems that Win takes sleep as an I/O operation)
def my_sleep(secs):
	try:
		sleep(secs)
	except IOError:
		pass

class sat:
	def __init__(self):
		# default settings
		self.use_conn_timeout = 1
		self.connect_timeout = 10
		self.recieve_timeout = 10
		self.log_recv_timeout = 1
		self.send_data = 1
		self.wait_data = 0.5
		self.data = "Can u suck my dick ?\n\n\n\n\r\n\r\n\r"
		self.case_sensitive = 1
		self.notify_all = 0
		self.notify_found = 1
		self.log_found = 1
		self.read_data = 15
		self.use_threads = 1
		self.max_threads = 100
		self.log_file = "sat.log"
		self.restore_file = "sat.restore"
		self.rules_debug = 0
		self.log_rules_debug = 1
		self.search_unknown = 0
		self.notify_unknown = 0
		self.log_unknown = 1
		# default rule
		self.srules = [["Default rule (everything)", 1, [1, "*"]]]
		# other vars (do not change)
		self.config_file = "sat.conf"
		self.rules_file = "sat.srules"
		self.all = 0
		self.found = 0
		self.unknown = 0
		self.cnt_thrds = 0 # especially don't change this or u will get neverending scans :)
		self.scan_type = "TCP"
		self.log_fd = ""
		self.die = 0
		self.queue_end = 0

	# this will execute the config file and check the settings there
	def exec_config(self, cfile):
		try:
			execfile(cfile)
		except IOError, error:
			if error[0] == 2:
				print "CONFIG FILE WARNING - No such file, useing the default settings"
			else:
				print "CONFIG FILE ERROR - %s" % error[1]
				exit()
		if type(self.recieve_timeout) not in [int, long, float] or self.recieve_timeout < 0:
			print "CONFIG ERROR - Recieve timeout must be a non-negative number"
			exit()
		if type(self.connect_timeout) not in [int, long, float] or self.connect_timeout < 0:
			print "CONFIG ERROR - Connect timeout must be a non-negative number"
			exit()
		if type(self.wait_data) not in [int, long, float] or self.wait_data < 0:
			print "CONFIG ERROR - Variable 'self.wait_data' must be a non-negative number"
			exit()
		if type(self.read_data) not in [int, long] or self.read_data < 0:
			print "CONFIG ERROR - Variable 'self.read_data' must be a non-negative number"
			exit()
		if type(self.max_threads) not in [int, long]:
			print "CONFIG ERROR - Variable 'self.max_threads' must be a number"
			exit()
		elif self.max_threads <= 1:
			self.use_threads = 0

	def exec_db(self, dbfile):
		# executing the rules file
		try:
			execfile(dbfile)
		except IOError, error:
			if error[0] == 2:
				print "RULES FILE WARNING - No such file, useing the default rule"
			else:
				print "RULES FILE ERROR - %s" % error[1]
				exit()
		except:
			print "RULES FILE ERROR - File is corrupted"
			exit()
		# checking the rules
		else:
			stdout.write("checking the rules.. ")
			stdout.flush()
			if type(self.srules) != list:
				print "RULES ERROR - Invalid rules file format detected"
				exit()
			for item in self.srules:
				if len(item) < 3 or type(item) != list:
					print "RULES ERROR - Invalid rule detected (%s. rule: %s)" % (self.srules.index(item), item)
					exit()
				for i in item[2:]:
					if len(i) < 2 or type(i) != list:
						print "RULES ERROR - Invalid rule detected (%s. rule: %s)" % (self.srules.index(item), item)
						exit()
			stdout.write("done%s" % linesep)
		stdout.write("removeing disabled rules and compiling the patterns.. ")
		stdout.flush()
		srules2 = []
		for rule in self.srules:
			if self.search_unknown or rule[1]:
				for subrule in rule[2:]:
					if not self.search_unknown and not subrule[0]:
						rule.remove(subrule)
					else:
						subrule2 = translate(subrule[1])
						if self.case_sensitive:
							rule[rule.index(subrule)] = [subrule[0], re.compile(subrule2, re.DOTALL)]
						else:
							rule[rule.index(subrule)] = [subrule[0], re.compile(subrule2, re.DOTALL | re.IGNORECASE)]
				if len(rule) > 2:
					srules2.append(rule)
			if self.die:
				exit()
		self.srules = srules2
		stdout.write("done%s" % linesep)

	def log_message(self, message):
		self.lock_log_file.acquire()
		self.log_fd.write(message)
		try:
			self.log_fd.flush()
		except IOError, error:
			print "RESULT FILE WARNING - %s" % error
		self.lock_log_file.release()

	def start_scan(self):
		self.exec_config(self.config_file)
		self.cmdline()
		self.exec_db(self.rules_file)
		if self.use_conn_timeout:
			socket.setdefaulttimeout(self.connect_timeout)
		self.lock_log_file = threading.Lock()
		self.lock_cnt_thrds = threading.Lock()
		self.lock_found = threading.Lock()
		self.lock_unknown = threading.Lock()
		self.log_fd = open_file(self.log_file, "a")
		if type(self.log_fd) != file:
			print "RESULT FILE WARNING - %s" % self.log_fd
		if self.scan_type == "TCP":
			self.try_connect = self.try_connect_tcp
		elif self.scan_type == "UDP":
			self.try_connect = self.try_connect_udp
		if self.use_threads:
			self.data_queue = Queue.Queue(0)
			thrd = threading.Thread(target=self.get_from_queue)
			thrd.start()
			self.lock_cnt_thrds.acquire()
			self.cnt_thrds += 1
			self.lock_cnt_thrds.release()
		self.start_time = time()
		# 45 x '='
		self.log_message("=============================================%sscan started at %s%s" % (linesep, strftime("%H:%M:%S %d.%m.%Y", localtime(self.start_time)), linesep))
		print "scan started at %s" % strftime("%H:%M:%S %d.%m.%Y", localtime(self.start_time))
		print "scanning %s ports" % self.scan_type
		if self.type == "t":
			self.log_message("  using target file: %s%s  using port range : %s%s  scan type        : %s%s  ----%s" % (argv[argv.index("-t")+1], linesep, argv[argv.index("-p")+1], linesep, self.scan_type, linesep, linesep))
			self.ipset = "-"
			if len(self.targets) == 1 and len(self.ports) == 1 and "-" not in self.ports[0]:
				self.use_threads = 0
				self.queue_end = 1
				self.start_connecting(self.targets[0])
			else:
				for target in self.targets:
					self.start_connecting(target)
		elif self.type == "i":
			self.log_message("  using IP range  : %s%s  using port range: %s%s  scan type       : %s%s  ----%s" % (argv[argv.index("-i")+1], linesep, argv[argv.index("-p")+1], linesep, self.scan_type, linesep, linesep))
			if len(self.ips) == 1 and "-" not in self.ips[0] and len(self.ports) == 1 and "-" not in self.ports[0]:
				self.ipset = argv[argv.index("-i")+1]
				self.use_threads = 0
				self.queue_end = 1
				self.start_connecting(self.ips[0])
			else:
				for self.ipset in self.ips:
					if "-" in self.ipset:
						start_ip = self.ipset.replace("-", " ", 1).split()[0].replace(".", " ", 3).split()
						end_ip = self.ipset.replace("-", " ", 1).split()[1].replace(".", " ", 3).split()
						for i in xrange(4):
							start_ip[i] = int(start_ip[i])
							end_ip[i] = int(end_ip[i])
						# this is ugly but very fast
						for start_ip[0] in xrange(start_ip[0], 256):
							for start_ip[1] in xrange(start_ip[1], 256):
								for start_ip[2] in xrange(start_ip[2], 256):
									for start_ip[3] in xrange(start_ip[3], 256):
										if not self.die:
											self.start_connecting("%s.%s.%s.%s" % (start_ip[0], start_ip[1], start_ip[2], start_ip[3]))
										else:
											start_ip = end_ip
										if start_ip[0] == end_ip[0] and start_ip[1] == end_ip[1] and start_ip[2] == end_ip[2] and start_ip[3] == end_ip[3]:
											break
									if start_ip[0] == end_ip[0] and start_ip[1] == end_ip[1] and start_ip[2] == end_ip[2]:
										break
									start_ip[3] = 0
								if start_ip[0] == end_ip[0] and start_ip[1] == end_ip[1]:
									break
								start_ip[2] = 0
							if start_ip[0] == end_ip[0]:
								break
							start_ip[1] = 0
					else:
						self.start_connecting(self.ipset)
		if self.use_threads:
			stdout.write("waiting for all threads to complete.. ")
			stdout.flush()
			while self.cnt_thrds != 1:
				my_sleep(1)
			stdout.write("done%s" % linesep)
			self.queue_end = 1
			stdout.write("empting queue.. ")
			stdout.flush()
			while self.cnt_thrds != 0:
				my_sleep(1)
			stdout.write("done%s" % linesep)
		if self.die:
			try:
				file_obj = open_file(self.restore_file, "w")
				if type(file_obj) != file:
					print "RESTORE FILE WARNING - %s" % file_obj
				else:
					file_obj.write("%s %s %s %s %s" % (self.type, argv[argv.index("-%s" % self.type)+1], self.target, self.ipset, argv[argv.index("-p")+1]))
					try:
						file_obj.close()
					except IOError, error:
						print "RESTORE FILE WARNING - %s" % error
			except AttributeError:
				exit()
		self.log_message("  ----%s  scan time             : %0.4f s%s" % (linesep, time()-self.start_time, linesep))
		if self.use_threads:
			self.log_message("  average time          : %0.4f s for host/IP:port%s" % ((time()-self.start_time)/self.all, linesep))
		self.log_message("  --%s  scaned [host/IP:port]s: %s%s  found                 : %s%s" % (linesep, self.all, linesep, self.found, linesep))
		if self.search_unknown:
			self.log_message("  unknown               : %s%s" % (self.unknown, linesep))
		if self.die:
			self.log_message("scan aborted at %s%s" % (strftime("%H:%M:%S %d.%m.%Y"), linesep))
		else:
			self.log_message("scan completed at %s%s" % (strftime("%H:%M:%S %d.%m.%Y"), linesep))
		self.log_fd.close()
		print "----"
		print "scan time             : %0.4f s" % (time()-self.start_time)
		if self.use_threads:
			print "average time          : %0.4f s for host/IP:port" % ((time()-self.start_time)/self.all)
		print "--"
		print "scaned [host/IP:port]s: %s" % self.all
		print "found                 : %s" % self.found
		if self.search_unknown:
			print "unknown               : %s" % self.unknown
		print "----"
		if self.die:
			print "scan aborted at %s" % strftime("%H:%M:%S %d.%m.%Y")
		else:
			print "scan completed at %s" % strftime("%H:%M:%S %d.%m.%Y")

	def start_connecting(self, target):
		self.target = target
		for portset in self.ports:
			if "-" in portset:
				if self.use_threads:
					for port in xrange(int(portset.replace("-", " ", 1).split()[0]), int(portset.replace("-", " ", 1).split()[1])+1):
						thrd = threading.Thread(target=self.try_connect, args=(target, port,))
						while 1:
							if self.cnt_thrds < self.max_threads:
								try:
									self.lock_cnt_thrds.acquire()
									self.cnt_thrds += 1
									self.all += 1
									self.lock_cnt_thrds.release()
									thrd.start()
								except ThreadError, error:
									self.lock_cnt_thrds.acquire()
									self.cnt_thrds -= 1
									self.all -= 1
									self.lock_cnt_thrds.release()
									print "THREADS WARNING - Cannot create new thread: %s" % error
									my_sleep(1)
								else:
									break
							else:
								my_sleep(1)
						if self.die:
							break
				else:
					for port in xrange(int(portset.replace("-", " ", 1).split()[0]), int(portset.replace("-", " ", 1).split()[1])+1):
						self.all += 1
						self.try_connect(target, port)
						if self.die:
							break
			else:
				if self.use_threads:
					thrd = threading.Thread(target=self.try_connect, args=(target, portset,))
					while 1:
						if self.cnt_thrds < self.max_threads:
							try:
								self.lock_cnt_thrds.acquire()
								self.cnt_thrds += 1
								self.all += 1
								self.lock_cnt_thrds.release()
								thrd.start()
							except ThreadError, error:
								self.lock_cnt_thrds.acquire()
								self.cnt_thrds -= 1
								self.all -= 1
								self.lock_cnt_thrds.release()
								print "THREADS WARNING - Cannot create new thread: %s" % error
								my_sleep(1)
							else:
								break
						else:
							my_sleep(1)
				else:
					self.all += 1
					self.try_connect(target, portset)
			if self.die:
				break

	def try_connect_tcp(self, target, port):
		again = 1
		while again:
			again = 0
			try:
				if self.notify_all:
					print "TRYING      : %s:%s.." % (target, port)
				sck = telnetlib.Telnet(str(target), port)
			except socket.timeout:
				if self.notify_all:
					print "TIMED OUT   : %s:%s - Connect timeout" % (target, port)
			except socket.error, error:
				# 10050 - Network is down
				# 10051 - Network is unreachable
				if error[0] == 10050 or error[0] == 10051:
					again = 1
				if self.notify_all:
					print "ERROR       : %s:%s - %s" % (target, port, error[1])
			except socket.gaierror, error:
				if self.notify_all:
					print "ERROR       : %s:%s - %s" % (target, port, error[1])
			else:
				if self.notify_all:
					print "CONNECTED TO: %s:%s" % (target, port)
				try:
					if self.send_data:
						my_sleep(self.wait_data)
						sck.get_socket().send(self.data)
					rd, wr, ex = select.select([sck.fileno()], [], [], self.recieve_timeout)
					if rd:
						data = sck.read_some()
						for i in xrange(self.read_data):
							rd, wr, ex = select.select([sck.fileno()], [], [], 0.1)
							if rd:
								data = str(data)+str(sck.get_socket().recv(100))
						if self.use_threads:
							self.data_queue.put([target, port, data])
						else:
							self.match_rule(target, port, data)
					else:
						if self.notify_all:
							print "TIMED OUT   : %s:%s - Recieve timeout" % (target, port)
							if self.log_recv_timeout:
								self.log_message("  %s:%s = recieve timeout%s" % (target, port, linesep))
				except select.error, error:
					if self.notify_all:
						print "ERROR       : %s:%s - %s" % (target, port, error[1])
				except socket.error, error:
					if self.notify_all:
						print "ERROR       : %s:%s - %s" % (target, port, error[1])
				sck.close()
		if self.use_threads:
			self.lock_cnt_thrds.acquire()
			self.cnt_thrds -= 1
			self.lock_cnt_thrds.release()

	def try_connect_udp(self, target, port):
		again = 1
		while again:
			again = 0
			try:
				if self.notify_all:
					print "TRYING      : %s:%s.." % (target, port)
				sck = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
				sck.sendto(self.data, (str(target), int(port)))
			except socket.error, error:
				# 10050 - Network is down
				# 10051 - Network is unreachable
				if error[0] == 10050 or error[0] == 10051:
					again = 1
				if self.notify_all:
					print "ERROR       : %s:%s - %s" % (target, port, error[1])
			except socket.gaierror, error:
				if self.notify_all:
					print "ERROR       : %s:%s - %s" % (target, port, error[1])
			else:
				if self.notify_all:
					print "DATA SENT TO: %s:%s" % (target, port)
				try:
					if self.send_data:
						my_sleep(self.wait_data)
						sck.sendto(self.data, (target, int(port)))
					rd, wr, ex = select.select([sck.fileno()], [], [], self.recieve_timeout)
					if rd:
						data = sck.recvfrom(1000)[0]
						for i in xrange(self.read_data):
							rd, wr, ex = select.select([sck.fileno()], [], [], 0.1)
							if rd:
								data = str(data)+str(sck.recvfrom(100)[0])
						if self.use_threads:
							self.data_queue.put([target, port, data])
						else:
							self.match_rule(target, port, data)
					else:
						if self.notify_all:
							print "TIMED OUT   : %s:%s - UDP recieve timeout" % (target, port)
				except select.error, error:
					if self.notify_all:
						print "ERROR       : %s:%s - %s" % (target, port, error[1])
				except socket.error, error:
					if self.notify_all:
						print "ERROR       : %s:%s - %s" % (target, port, error[1])
				sck.close()
		if self.use_threads:
			self.lock_cnt_thrds.acquire()
			self.cnt_thrds -= 1
			self.lock_cnt_thrds.release()

	def get_from_queue(self):
		while not (self.queue_end and self.data_queue.empty()):
				if not self.data_queue.empty():
					item = self.data_queue.get()
					self.match_rule(item[0], item[1], item[2])
				else:
					my_sleep(1)
		self.lock_cnt_thrds.acquire()
		self.cnt_thrds -= 1
		self.lock_cnt_thrds.release()

	def match_rule(self, target, port, data):
		if self.rules_debug:
			print "DEBUG       : %s:%s - %s" % (target, port, [data])
			if self.log_rules_debug:
				self.log_message("  %s:%s = %s%s" % (target, port, [data], linesep))
		found = ""
		for rule in self.srules:
			for subrule in rule[2:]:
				if subrule[1].match(data) is not None:
					found = str(found)+str(rule[0])+" | "
					break
		if found != "":
			self.lock_found.acquire()
			self.found += 1
			self.lock_found.release()
			if self.notify_all or self.notify_found:
				print "FOUND       : %s:%s (%s)" % (target, port, found[0:-3])
			if self.log_found:
				self.log_message("  %s:%s = %s%s" % (target, port, found[0:-3], linesep))
		elif self.search_unknown:
			self.lock_unknown.acquire()
			self.unknown += 1
			self.lock_unknown.release()
			if self.notify_unknown:
				print "UNKNOWN     : %s:%s (%s)" % (target, port, [data])
			if self.log_unknown:
				self.log_message("  %s:%s = unknown - %s%s" % (target, port, [data], linesep))
		if self.notify_all and not self.use_threads:
			print ""

	# this function is very ugly but works well ;)
	# ..and will be rewrited soon
	def cmdline(self):
		hove = 0
		fefet = 1
		while fefet != len(argv):
			if argv[fefet] == "-t":
				if len(argv) > fefet+1:
					file_obj = open_file(argv[fefet+1], "r")
					if type(file_obj) != file:
						print "TARGET FILE ERROR - %s" % file_obj
						exit()
					targets = file_obj.readlines()
					file_obj.close()
					self.targets = []
					for i in xrange(len(targets)):
						if targets[i][0] != "#":
							targets[i] = targets[i].replace("\r", "").replace("\n", "")
							if targets[i] != "":
								self.targets.append(targets[i])
					if self.targets == []:
						print "TARGETS ERROR - No targets loaded"
						exit()
					self.type = "t"
					self.ipset = "-"
					fefet += 2
					hove += 2
				else:
					print "CMDLINE ERROR - Option '-t': Missing argument"
					exit()
			elif argv[fefet] == "-i":
				if len(argv) > fefet+1:
					self.ips = argv[fefet+1].split(",")
					if len(self.ips) != 1 or "-" in self.ips[0]:
						for ipset in self.ips:
							if "-" in ipset:
								iprange = ipset.split("-")
								if len(iprange) != 2:
									print "CMDLINE ERROR - Option '-i': Invalid argument"
									exit()
								if not (test_ipv4(iprange[0]) and test_ipv4(iprange[1])):
									print "CMDLINE ERROR - Option '-i': Invalid argument"
									exit()
								iprange[0] = iprange[0].split(".")
								iprange[1] = iprange[1].split(".")
								for i in xrange(4):
									iprange[0][i] = int(iprange[0][i])
									iprange[1][i] = int(iprange[1][i])
								start_ip = iprange[0][0] * 256**3 + iprange[0][1] * 256**2 + iprange[0][2] * 256 + iprange[0][3]
								end_ip = iprange[1][0] * 256**3 + iprange[1][1] * 256**2 + iprange[1][2] * 256 + iprange[1][3]
								if start_ip > end_ip:
									print "CMDLINE ERROR - Option '-i': Invalid argument"
									exit()
							#else:
							#	if not test_ipv4(ipset):
							#		print "CMDLINE ERROR - Option '-i': Invalid argument"
							#		exit()
					self.type = "i"
					fefet += 2
					hove += 2
				else:
					print "CMDLINE ERROR - Option '-i': Missing argument"
					exit()
			elif argv[fefet] == "-r":
				if len(argv) == 3:
					restore = argv[fefet+1]
				elif len(argv) == 2:
					restore = self.restore_file
				else:
					hove = 0
					break
				file_obj = open_file(restore, "r")
				if type(file_obj) != file:
					print "RESTORE FILE ERROR - %s" % file_obj
					exit()
				line = file_obj.readline()
				if len(line.split()) < 4:
					print "RESTORE FILE ERROR - File is corrupted"
					exit()
				if line.split()[0] == "t":
					file_obj = open_file(line.split()[1], "r")
					if type(file_obj) != file:
						print "TARGET FILE ERROR - %s" % file_obj
						exit()
					targets = file_obj.readlines()
					file_obj.close()
					self.targets = []
					for i in xrange(len(targets)):
						if targets[i][0] != "#":
							targets[i] = targets[i].replace("\r", "").replace("\n", "")
							if targets[i] != "":
								self.targets.append(targets[i])
					while len(self.targets) != 0:
						if self.targets[0] != line.split()[2]:
							self.targets.pop(0)
						else:
							break
					if self.targets == []:
						print "RESTORE ERROR - Can't restore scan (no targets loaded)"
						exit()
				elif line.split()[0] == "i":
					self.ips = line.split()[1].split(",")
					while len(self.ips) != 0:
						if self.ips[0] != line.split()[3]:
							self.ips.pop(0)
						else:
							break
					if len(self.ips) == 0:
						print "RESTORE ERROR - Can't restore scan (no targets loaded)"
						exit()
					if "-" in self.ips[0]:
						self.ips[0] = "%s-%s" % (line.split()[2], self.ips[0].split("-")[1])
					for ipset in self.ips:
						if "-" in ipset:
							iprange = ipset.split("-")
							if len(iprange) != 2:
								print "RESTORE FILE ERROR - File is corrupted"
								exit()
							if not (test_ipv4(iprange[0]) and test_ipv4(iprange[1])):
								print "RESTORE FILE ERROR - File is corrupted"
								exit()
							iprange[0] = iprange[0].split(".")
							iprange[1] = iprange[1].split(".")
							for i in xrange(4):
								iprange[0][i] = int(iprange[0][i])
								iprange[1][i] = int(iprange[1][i])
							start_ip = iprange[0][0] * 256**3 + iprange[0][1] * 256**2 + iprange[0][2] * 256 + iprange[0][3]
							end_ip = iprange[1][0] * 256**3 + iprange[1][1] * 256**2 + iprange[1][2] * 256 + iprange[1][3]
							if start_ip > end_ip:
								print "RESTORE FILE ERROR - File is corrupted"
								exit()
				else:
					print "RESTORE FILE ERROR - File is corrupted"
					exit()
				self.ports = line.split()[4].split(",")
				for portset in self.ports:
					if "-" in portset:
						try:
							if int(portset.split("-")[0]) < 0 or int(portset.split("-")[0]) > 65535:
								print "RESTORE FILE ERROR - File is corrupted"
								exit()
							if int(portset.split("-")[1]) < 0 or int(portset.split("-")[1]) > 65535:
								print "RESTORE FILE ERROR - File is corrupted"
								exit()
							if int(portset.split("-")[0]) > int(portset.split("-")[1]):
								print "RESTORE FILE ERROR - File is corrupted"
								exit()
						except ValueError:
							print "RESTORE FILE ERROR - File is corrupted"
							exit()
					else:
						try:
							if int(portset) < 0 or int(portset) > 65535:
								print "RESTORE FILE ERROR - File is corrupted"
								exit()
						except ValueError:
							print "RESTORE FILE ERROR - File is corrupted"
							exit()
				self.type = line.split()[0]
				argv.append("-%s" % line.split()[0])
				argv.append(line.split()[1])
				argv.append("-p")
				argv.append(line.split()[4])
				hove = 3
				break
			elif argv[fefet] == "-p":
				if len(argv) > fefet+1:
					self.ports = argv[fefet+1].split(",")
					for portset in self.ports:
						if "-" in portset:
							try:
								if int(portset.split("-")[0]) < 0 or int(portset.split("-")[0]) > 65535:
									print "CMDLINE ERROR - Option '-p': Port must be a number between 0 and 65535"
									exit()
								if int(portset.split("-")[1]) < 0 or int(portset.split("-")[1]) > 65535:
									print "CMDLINE ERROR - Option '-p': Port must be a number between 0 and 65535"
									exit()
								if int(portset.split("-")[0]) > int(portset.split("-")[1]):
									print "CMDLINE ERROR - Option '-p': Invalid port range"
									exit()
							except ValueError:
								print "CMDLINE ERROR - Option '-p': Port must be a number"
								exit()
						else:
							try:
								if int(portset) < 0 or int(portset) > 65535:
									print "CMDLINE ERROR - Option '-p': Port must be a number between 0 and 65535"
									exit()
							except ValueError:
								print "CMDLINE ERROR - Option '-p': Port must be a number"
								exit()
					fefet += 2
					hove += 1
				else:
					print "CMDLINE ERROR - Option '-p': Missing argument"
					exit()
			elif argv[fefet] == "-c":
				if len(argv) > fefet+1:
					#self.exec_config(argv[fefet+1])
					self.config_file = argv[fefet+1]
					fefet += 2
				else:
					print "CMDLINE ERROR - Option '-c': Missing argument"
					exit()
			elif argv[fefet] == "-l":
				if len(argv) > fefet+1:
					#self.exec_db(argv[fefet+1])
					self.rules_file = argv[fefet+1]
					fefet += 2
				else:
					print "CMDLINE ERROR - Option '-l': Missing argument"
					exit()
			elif argv[fefet] == "-T":
				if len(argv) > fefet+1:
					try:
						argv[fefet+1] = int(argv[fefet+1])
					except ValueError:
						print "CMDLINE ERROR - Option '-T': Argument must be a number bigger then 0"
						exit()
					if argv[fefet+1] <= 1:
						self.use_threads = 0
					else:
						self.use_threads = 1
						self.max_threads = argv[fefet+1]
					fefet += 2
				else:
					print "CMDLINE ERROR - Option '-T': Missing argument"
					exit()
			elif argv[fefet] == "-U":
				self.scan_type = "UDP"
				fefet += 1
			elif argv[fefet] == "-d":
				if len(argv) > fefet+1:
					if str(argv[fefet+1]) == "1":
						self.rules_debug = 1
						self.log_rules_debug = 0
					elif str(argv[fefet+1]) == "2":
						self.rules_debug = 1
						self.log_rules_debug = 1
					else:
						print "CMDLINE ERROR - Option '-d': Invalid argument"
						exit()
					fefet += 2
				else:
					print "CMDLINE ERROR - Option '-d': Missing argument"
					exit()
			elif argv[fefet] == "-h":
				fefet = len(argv)
				hove = 0
			else:
				print "CMDLINE ERROR - Unknown option '%s'" % argv[fefet]
				exit()
		if hove != 3:
			print "Usage:"
			print " %s <scan type> -p <port_range> [options]" % argv[0]
			print " %s -r [file]" % argv[0]
			print " %s -h" % argv[0]
			print ""
			print " Scan types:"
			print " -t <target_file> - specify the target file"
			print " -i <ip_range>    - ip range to scan, example: 10.1.1.1-10.1.1.2,10.1.2.2"
			print ""
			print " Options:"
			print " -U               - scan UDP ports rather then TCP"
			print " -T <number>      - maximum number of threads"
			print " -c <file>        - specify the config file"
			print " -l <file>        - specify the rules file"
			print " -d <level>       - rules debug mode, level can be 1 or 2"
			print ""
			print " Other switches:"
			print " -p <port_range>  - port range to scan, example: 1-1024,3333,4000-5000"
			print " -r [file]        - restore aborted scan"
			print " -h               - print this help and exit"
			exit()

if __name__ == "__main__":
	signal.signal(signal.SIGINT, sig_hand)
	signal.signal(signal.SIGTERM, sig_hand)
	print "::   SAT (Simple scAnning Tool)   ::"
	print "::          version 1.0           ::"
	print "::     http://sat.berlios.de/     ::"
	print "::   Copyright (C) 2005 azurIt    ::"
	print ":: azurit@pobox.sk, azurIt@IRCnet ::"
	sat = sat()
	sat.start_scan()
